@@ -72,11 +72,14 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
     def test_string_representation(self):
         parser = self.parser.__class__()
-        self.assertEqual(repr(parser), f"{parser.__class__.__name__}()")
+        self.assertEqual(
+            repr(parser), f"<{parser.__class__.__name__} object at {hex(id(parser))}>"
+        )
+        self.assertEqual(str(parser), f"{parser.__class__.__name__}()")
 
         parser = self.parser.__class__(namespaces={'tst': 'http://xpath.test/ns'})
         self.assertEqual(
-            repr(parser), f"{parser.__class__.__name__}({{'tst': 'http://xpath.test/ns'}})"
+            str(parser), f"{parser.__class__.__name__}({{'tst': 'http://xpath.test/ns'}})"
         )
 
     def test_parser_pickling(self):
@@ -136,32 +139,38 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
     def test_token_classes(self):
         # Literals
-        self.check_token('(string)', 'literal', "'hello' string",
-                         f"_StringLiteral({self.parser!r}, 'hello')", 'hello')
-        self.check_token('(integer)', 'literal', "1999 integer",
-                         f"_IntegerLiteral({self.parser!r}, 1999)", 1999)
-        self.check_token('(float)', 'literal', "3.1415 float",
-                         f"_FloatLiteral({self.parser!r}, 3.1415)", 3.1415)
-        self.check_token('(decimal)', 'literal', "217.35 decimal",
-                         f"_DecimalLiteral({self.parser!r}, 217.35)", 217.35)
-        self.check_token('(name)', 'literal', "'schema' name",
-                         f"_NameLiteral({self.parser!r}, 'schema')", 'schema')
+        self.check_token(
+            '(string)', 'literal', "'hello' string",
+            value='hello'
+        )
+        self.check_token(
+            '(integer)', 'literal', "1999 integer",
+            value=1999
+        )
+        self.check_token(
+            '(float)', 'literal', "3.1415 float",
+            value=3.1415
+        )
+        self.check_token(
+            '(decimal)', 'literal', "217.35 decimal",
+            value=217.35
+        )
+        self.check_token(
+            '(name)', 'literal', "'schema' name",
+            value='schema'
+        )
 
         # Variables
-        self.check_token('$', 'operator', "$ variable reference",
-                         f"_DollarSignOperator({self.parser!r})")
+        self.check_token('$', 'operator', "$ variable reference")
 
         # Axes
-        self.check_token('self', 'axis', "'self' axis",
-                         f"_SelfAxis({self.parser!r})")
-        self.check_token('child', 'axis', "'child' axis",
-                         f"_ChildAxis({self.parser!r})")
-        self.check_token('parent', 'axis', "'parent' axis",
-                         f"_ParentAxis({self.parser!r})")
-        self.check_token('ancestor', 'axis', "'ancestor' axis",
-                         f"_AncestorAxis({self.parser!r})")
-        self.check_token('preceding', 'axis', "'preceding' axis",
-                         expected_repr=f"_PrecedingAxis({self.parser!r})")
+        self.check_token('self', 'axis', "'self' axis")
+        self.check_token('child', 'axis', "'child' axis")
+        self.check_token('parent', 'axis', "'parent' axis")
+        self.check_token('ancestor', 'axis', "'ancestor' axis")
+        self.check_token(
+            'preceding', 'axis', "'preceding' axis"
+        )
         self.check_token('descendant-or-self', 'axis', "'descendant-or-self' axis")
         self.check_token('following-sibling', 'axis', "'following-sibling' axis")
         self.check_token('preceding-sibling', 'axis', "'preceding-sibling' axis")
@@ -176,22 +185,15 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_token(
             symbol='position',
             expected_label='function',
-            expected_str="'position' function",
-            expected_repr=f"_PositionFunction({self.parser!r})"
+            expected_str="'fn:position' function",
         )
 
         # Operators
-        self.check_token(
-            'and', 'operator', "'and' operator", f"_AndOperator({self.parser!r})"
-        )
+        self.check_token('and', 'operator', "'and' operator")
         if self.parser.version == '1.0':
-            self.check_token(
-                ',', 'symbol', "comma symbol", f"_CommaSymbol({self.parser!r})"
-            )
+            self.check_token(',', 'symbol', "comma symbol")
         else:
-            self.check_token(
-                ',', 'operator', "comma operator", f"_CommaOperator({self.parser!r})"
-            )
+            self.check_token(',', 'operator', "comma operator")
 
     def test_token_tree(self):
         self.check_tree('child::B1', '(child (B1))')
@@ -423,7 +425,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             # Skip lxml test because lxml's XPath doesn't include document root
             self.check_selector("//self::node()", document, [document, root, 'Dickens'])
             self.check_selector("/self::node()", document, [document])
-            self.check_selector("/self::node()", root, [root])
+            self.check_selector("/self::node()", root, [])
 
         self.check_selector("//self::text()", root, ['Dickens'])
 
@@ -431,9 +433,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_select("node()", [context.root.getroot()], context)
 
         context = XPathContext(root)
-        context.item = None
-        # lxml differs: doesn't consider the document position even if select from an ElementTree
-        self.check_value("/self::node()", expected=[context.root], context=context)
+        self.check_value("/self::node()", expected=[], context=context)
 
         context.item = 1
         self.check_value("self::node()", expected=[], context=context)
@@ -820,7 +820,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("boolean('   ')", True)
         self.check_value("boolean('')", False)
 
-        self.wrong_type("true(1)", 'XPST0017', "'true' function has no arguments")
+        self.wrong_type("true(1)", 'XPST0017', "'fn:true' function has no arguments")
         self.wrong_syntax("true(", 'unexpected end of source')
 
         if self.parser.version == '1.0':
@@ -1129,7 +1129,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         root = self.etree.XML(XML_DATA_TEST)
         context = XPathContext(root, variables=self.variables)
         self.check_value("sum($values)", 35, context)
-        self.check_selector("sum(/values/a)", root, 13.299999999999999)
+        self.check_selector("sum(/values/a)", root, 13.3)
 
         if self.parser.version == '1.0':
             self.check_selector("sum(/values/*)", root, math.isnan)
@@ -1404,18 +1404,18 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
         if self.parser.version == '1.0':
             self.check_selector('/A/namespace::*', root, expected=set(namespaces),
-                                namespaces=namespaces[-1:])
+                                namespaces=dict(namespaces[-1:]))
             self.check_selector('/A/namespace::tst', root,
                                 expected=[('tst', 'http://xpath.test/ns')],
-                                namespaces=namespaces[-1:])
+                                namespaces=dict(namespaces[-1:]))
         else:
             self.check_selector('/A/namespace::*', root,
                                 expected={'http://www.w3.org/XML/1998/namespace',
                                           'http://xpath.test/ns'},
-                                namespaces=namespaces[-1:])
+                                namespaces=dict(namespaces[-1:]))
             self.check_selector('/A/namespace::tst', root,
                                 expected=['http://xpath.test/ns'],
-                                namespaces=namespaces[-1:])
+                                namespaces=dict(namespaces[-1:]))
 
         self.check_value('namespace::*', MissingContextError)
         self.check_value('./text()/namespace::*', [], context=XPathContext(root))
@@ -1424,7 +1424,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_selector('/A/namespace::namespace-node()', root,
                                 expected={'http://www.w3.org/XML/1998/namespace',
                                           'http://xpath.test/ns'},
-                                namespaces=namespaces[-1:])
+                                namespaces=dict(namespaces[-1:]))
 
     def test_parent_shortcut_and_axis(self):
         root = self.etree.XML(
@@ -1654,6 +1654,42 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
         self.check_selector("//target[name=//var/name]", root, expected=[root[0]])
 
+    def test_get_function(self):
+        func = self.parser.get_function('fn:true', 0)
+        self.assertEqual(str(func), "'fn:true' function")
+        self.assertTrue(repr(func).startswith('<XPathFunction fn:true#0 at 0x'))
+        self.assertTrue(func())
+
+        with self.assertRaises(NameError) as ctx:
+            self.parser.get_function('foo', 1)
+        self.assertIn("unknown function 'foo'", str(ctx.exception))
+
+        with self.assertRaises(TypeError) as ctx:
+            self.parser.get_function('fn:true', 1)
+        self.assertIn('unknown function fn:true#1', str(ctx.exception))
+
+        func = self.parser.get_function('fn:false', 0)
+        self.assertTrue(repr(func).startswith('<XPathFunction fn:false#0 at 0x'))
+        self.assertFalse(func())
+
+        func = self.parser.get_function('concat', 2)
+        self.assertTrue(repr(func).startswith('<XPathFunction fn:concat#2 at 0x'))
+        self.assertEqual(func('foo', 1), 'foo1')
+
+        with self.assertRaises(TypeError) as ctx:
+            func('foo')
+        self.assertIn('missing required arguments', str(ctx.exception))
+
+        with self.assertRaises(TypeError) as ctx:
+            func('foo', 1, 2)
+        self.assertIn('too many arguments', str(ctx.exception))
+
+        func = self.parser.get_function('concat', 4)
+        self.assertTrue(repr(func).startswith('<XPathFunction fn:concat#4 at 0x'))
+        self.assertEqual(func('foo', 1, ' bar', 2), 'foo1 bar2')
+
+        self.assertEqual(func.as_function()('foo', 1, ' bar', 2), 'foo1 bar2')
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath1ParserTest(XPath1ParserTest):
@@ -1675,6 +1711,17 @@ class LxmlXPath1ParserTest(XPath1ParserTest):
                     set(root.xpath(path, namespaces=namespaces, **variables)), expected
                 )
                 self.assertEqual(set(results), expected)
+            elif isinstance(expected, float):
+                if math.isnan(expected):
+                    self.assertTrue(math.isnan(
+                        root.xpath(path, namespaces=namespaces, **variables)
+                    ))
+                    self.assertTrue(math.isnan(results))
+                else:
+                    self.assertAlmostEqual(
+                        root.xpath(path, namespaces=namespaces, **variables), expected
+                    )
+                    self.assertAlmostEqual(results, expected)
             elif not callable(expected):
                 self.assertEqual(root.xpath(path, namespaces=namespaces, **variables), expected)
                 self.assertEqual(results, expected)
@@ -1690,14 +1737,14 @@ class LxmlXPath1ParserTest(XPath1ParserTest):
         namespaces += [('tst', 'http://xpath.test/ns')]
 
         self.check_selector('/A/namespace::*', root, expected=set(namespaces),
-                            namespaces=namespaces[-1:])
+                            namespaces=dict(namespaces[-1:]))
         self.check_selector('/A/namespace::*', root, expected=set(namespaces))
 
         root = self.etree.XML('<tst:A xmlns:tst="http://xpath.test/ns" '
                               'xmlns="http://xpath.test/ns"><B1/></tst:A>')
         namespaces.append((None, 'http://xpath.test/ns'))
         self.check_selector('/tst:A/namespace::*', root, set(namespaces),
-                            namespaces=namespaces[-2:-1])
+                            namespaces=dict(namespaces[-2:-1]))
 
     def test_issue_25_with_count_function(self):
         root = lxml_etree.fromstring("""
