@@ -43,12 +43,12 @@ else:
     xmlschema.XMLSchema.meta_schema.build()
 
 from elementpath import XPath2Parser, XPathContext, ElementPathError, \
-    MissingContextError, select, Selector, datatypes, AttributeNode, \
-    NamespaceNode, TextNode
+    MissingContextError, select, Selector, datatypes, get_node_tree, \
+    AttributeNode, NamespaceNode, TextNode
 from elementpath.namespaces import XSI_NAMESPACE, XML_NAMESPACE, XML_ID
 from elementpath.datatypes import DateTime10, DateTime, Date10, Date, Time, \
     Timezone, DayTimeDuration, YearMonthDuration, QName, UntypedAtomic
-from elementpath.xpath_token import UNICODE_CODEPOINT_COLLATION
+from elementpath.collations import UNICODE_CODEPOINT_COLLATION
 
 try:
     from tests import test_xpath1_parser
@@ -287,11 +287,12 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
             locale.setlocale(locale.LC_COLLATE, 'en_US.UTF-8')
             self.check_value("fn:compare('Strasse', 'Straße')", -1)
-            self.check_value("fn:compare('Strassen', 'Straße')", 1)
+            self.check_value("fn:compare('Strassen', 'Straße')", -1)
 
             try:
+                self.check_value("fn:compare('Strasse', 'Straße', "
+                                 "'http://www.w3.org/2013/collation/UCA?lang=it_IT.UTF-8')", -1)
                 self.check_value("fn:compare('Strasse', 'Straße', 'it_IT.UTF-8')", -1)
-                self.check_value("fn:compare('Strassen', 'Straße')", 1)
             except locale.Error:
                 pass  # Skip test if 'it_IT.UTF-8' is an unknown locale setting
 
@@ -1374,6 +1375,14 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('deep-equal($a, $a)', True, context=context)
         self.check_value('deep-equal($a, $b)', False, context=context)
 
+    def test_deep_equal_function_on_nested_sequences(self):
+        self.check_value('fn:deep-equal(1, 1)', True)
+        self.check_value('fn:deep-equal(1, (1))', True)
+        self.check_value('fn:deep-equal(1, (1, ()))', True)
+        self.check_value('fn:deep-equal(1, (1, (1)))', False)
+        self.check_value('fn:deep-equal((1, ()), (1, (1)))', False)
+        self.check_value('fn:deep-equal(((1), 1), (1, (1)))', True)
+
     def test_adjust_datetime_to_timezone_function(self):
         context = XPathContext(root=self.etree.XML('<A/>'), timezone=Timezone.fromstring('-05:00'),
                                variables={'tz': DayTimeDuration.fromstring("-PT10H")})
@@ -1520,7 +1529,16 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         document = self.etree.parse(io.StringIO('<A xml:base="/base_path/"/>'))
         context = XPathContext(root=document)
-        self.check_value('fn:document-uri(.)', '/base_path/', context=context)
+        self.check_value('fn:document-uri(.)', context=context)  # xml:base doesn't apply!
+
+        document_node = get_node_tree(document, uri='/foo/bar.xml')
+        context = XPathContext(root=document_node)
+        self.check_value('fn:document-uri(.)', '/foo/bar.xml', context=context)
+
+        # Relative URIs doesn't apply for fn:document-uri
+        document_node = get_node_tree(document, uri='foo/bar.xml')
+        context = XPathContext(root=document_node)
+        self.check_value('fn:document-uri(.)', None, context=context)
 
     def test_doc_functions(self):
         root = self.etree.XML("<A><B1><C1/></B1><B2/><B3/></A>")
